#!/usr/local/bin/perl
#########################################################################
# chknews.pl - Newsgroups nach neuen Titeln durchsuchen
#########################################################################
# Parameter

my $newsrc     = "$ENV{HOME}/news.dat";              # "Gedchtnis"
my $newsserver = "$ENV{NNTPSERVER}";                      # Newsserver

#########################################################################

use Net::NNTP;                              # NNTP User Agent

$once_a_day = 0;                            # Parameter fuer "Nur einmal
# tglich" gesetzt?
@ARGV = grep { !(/^-once$/ && ($once_a_day = 1)) } @ARGV;

if(open(FILE, "<$newsrc")) {                # Gespeicherte Daten einlesen
while(<FILE>) {
chop;
# Abbrechen, falls -once
if($once_a_day && (($last_check) = /^_last_check: (\d+)/)) {
exit 1 if $last_check eq (localtime(time))[7];
}

$subjects_old{$_} = 1;              # Subject setzen
}
close(FILE);
}

$| = 1;                                     # Ausgabe nicht puffern

$newsagent = Net::NNTP->new($newsserver);   # News Agent erzeugen

# Fr alle Newsgroup/Subject-
# Paare auf der Kommandozeile
while(($newsgroup, $regex) = splice(@ARGV, 0, 2)) {

usage("Argument Error") unless defined $regex;

print "Scan [$newsgroup] for [$regex] ... ";

# Verfgbare Artikelnummern 
# Newsgroup holen
my ($total, $first, $last) = $newsagent->group($newsgroup);
my $count = 0;                          # Treffer fr aktuelle Group

unless(defined $total) {                # Fehler?
usage("Unknown newsgroup $newsgroup");
}
# Subjects holen
$subjects = $newsagent->xhdr('Subject', [$first, $last]);

# Fehler?
die "xhdr failed" unless defined $subjects;

# ber alle gefundenen
# Subjects iterieren
foreach $mesgno (keys %{$subjects}) {
# Match des gefundenen Subjects
# auf spezifizierten Regex?
if($subjects->{$mesgno} =~ /$regex/i) {

$subjects_new{"$newsgroup $mesgno"} = 1; # Treffer speichern

# Ausgeben, falls noch nicht
# bekannt
unless($subjects_old{"$newsgroup $mesgno"}) {
print "\n" unless $count++;
print "    $subjects->{$mesgno}\n";
}
}
}

print "No News\n" unless $count;
}

$newsagent->quit();                             # Newsagent freigeben

# Persistente Daten
# wieder ablegen
open(FILE, ">$newsrc") || die "Cannot open file $newsrc";

foreach $i (keys %subjects_new) {
print FILE "$i\n";
}
# Heutiges Datum ablegen,
# falls -once spezifiziert 
print FILE "_last_check: ", (localtime(time))[7], "\n" if $once_a_day;

close(FILE);


#########################################################################
sub usage {                                     # Usage ausgeben
#########################################################################
(my $program = $0) =~ s,.*/,,g;
print <<EOT;
$program: @_

usage:
$program [-once] newsgroup subject [newsgroup2 subject2] ...
-once: check only once a day
EOT

exit 1;
}
